@@ -76,6 +76,7 @@ struct cma_device {
 	struct ibv_pd	   *pd;
 	uint64_t	    guid;
 	int		    port_cnt;
+	int		    max_qpsize;
 	uint8_t		    max_initiator_depth;
 	uint8_t		    max_responder_resources;
 };
@@ -267,13 +268,12 @@ int ucma_init(void)
 		}
 
 		cma_dev->port_cnt = attr.phys_port_cnt;
+		cma_dev->max_qpsize = attr.max_qp_wr;
 		cma_dev->max_initiator_depth = (uint8_t) attr.max_qp_init_rd_atom;
 		cma_dev->max_responder_resources = (uint8_t) attr.max_qp_rd_atom;
 		ib += (cma_dev->verbs->device->transport_type == IBV_TRANSPORT_IB);
 	}
 
-	if (ib)
-		ucma_ib_init();
 	cma_dev_cnt = dev_cnt;
 	ucma_set_af_ib_support();
 	pthread_mutex_unlock(&mut);
@@ -379,7 +379,7 @@ static void ucma_free_id(struct cma_id_private *id_priv)
 
 	if (id_priv->sync)
 		rdma_destroy_event_channel(id_priv->id.channel);
-	if (id_priv->connect)
+	if (id_priv->connect_len)
 		free(id_priv->connect);
 	free(id_priv);
 }
@@ -516,7 +516,7 @@ static int ucma_addrlen(struct sockaddr *addr)
 	case PF_INET6:
 		return sizeof(struct sockaddr_in6);
 	case PF_IB:
-		return sizeof(struct sockaddr_ib);
+		return af_ib_support ? sizeof(struct sockaddr_ib) : 0;
 	default:
 		return 0;
 	}
@@ -745,10 +745,12 @@ int rdma_bind_addr(struct rdma_cm_id *id, struct sockaddr *addr)
 	return ucma_query_route(id);
 }
 
-static int ucma_complete(struct cma_id_private *id_priv)
+int ucma_complete(struct rdma_cm_id *id)
 {
+	struct cma_id_private *id_priv;
 	int ret;
 
+	id_priv = container_of(id, struct cma_id_private, id);
 	if (!id_priv->sync)
 		return 0;
 
@@ -761,8 +763,14 @@ static int ucma_complete(struct cma_id_private *id_priv)
 	if (ret)
 		return ret;
 
-	if (id_priv->id.event->status)
-		ret = ERR(id_priv->id.event->status);
+	if (id_priv->id.event->status) {
+		if (id_priv->id.event->event == RDMA_CM_EVENT_REJECTED)
+			ret = ERR(ECONNREFUSED);
+		else if (id_priv->id.event->status < 0)
+			ret = ERR(-id_priv->id.event->status);
+		else
+			ret = ERR(-id_priv->id.event->status);
+	}
 	return ret;
 }
 
@@ -788,7 +796,7 @@ static int rdma_resolve_addr2(struct rdma_cm_id *id, struct sockaddr *src_addr,
 		return (ret >= 0) ? ERR(ENODATA) : -1;
 
 	memcpy(&id->route.addr.dst_addr, dst_addr, dst_len);
-	return ucma_complete(id_priv);
+	return ucma_complete(id);
 }
 
 int rdma_resolve_addr(struct rdma_cm_id *id, struct sockaddr *src_addr,
@@ -803,6 +811,9 @@ int rdma_resolve_addr(struct rdma_cm_id *id, struct sockaddr *src_addr,
 		return ERR(EINVAL);
 
 	src_len = ucma_addrlen(src_addr);
+	if (src_addr && !src_len)
+		return ERR(EINVAL);
+
 	if (af_ib_support)
 		return rdma_resolve_addr2(id, src_addr, src_len, dst_addr,
 					  dst_len, timeout_ms);
@@ -820,7 +831,7 @@ int rdma_resolve_addr(struct rdma_cm_id *id, struct sockaddr *src_addr,
 		return (ret >= 0) ? ERR(ENODATA) : -1;
 
 	memcpy(&id->route.addr.dst_addr, dst_addr, dst_len);
-	return ucma_complete(id_priv);
+	return ucma_complete(id);
 }
 
 static int ucma_set_ib_route(struct rdma_cm_id *id)
@@ -840,8 +851,12 @@ static int ucma_set_ib_route(struct rdma_cm_id *id)
 	if (ret)
 		return ret;
 
-	ret = rdma_set_option(id, RDMA_OPTION_IB, RDMA_OPTION_IB_PATH,
-			      rai->ai_route, rai->ai_route_len);
+	if (rai->ai_route_len)
+		ret = rdma_set_option(id, RDMA_OPTION_IB, RDMA_OPTION_IB_PATH,
+				      rai->ai_route, rai->ai_route_len);
+	else
+		ret = -1;
+
 	rdma_freeaddrinfo(rai);
 	return ret;
 }
@@ -868,7 +883,7 @@ int rdma_resolve_route(struct rdma_cm_id *id, int timeout_ms)
 		return (ret >= 0) ? ERR(ENODATA) : -1;
 
 out:
-	return ucma_complete(id_priv);
+	return ucma_complete(id);
 }
 
 static int ucma_is_ud_qp(enum ibv_qp_type qp_type)
@@ -1084,10 +1099,10 @@ static void ucma_destroy_cqs(struct rdma_cm_id *id)
 	if (id->recv_cq_channel)
 		ibv_destroy_comp_channel(id->recv_cq_channel);
 
-	if (id->send_cq)
+	if (id->send_cq && (id->send_cq != id->recv_cq))
 		ibv_destroy_cq(id->send_cq);
 
-	if (id->send_cq_channel)
+	if (id->send_cq_channel && (id->send_cq_channel != id->recv_cq_channel))
 		ibv_destroy_comp_channel(id->send_cq_channel);
 }
 
@@ -1319,12 +1334,12 @@ int rdma_connect(struct rdma_cm_id *id, struct rdma_conn_param *conn_param)
 	if (ret != sizeof cmd)
 		return (ret >= 0) ? ERR(ENODATA) : -1;
 
-	if (id_priv->connect) {
+	if (id_priv->connect_len) {
 		free(id_priv->connect);
 		id_priv->connect_len = 0;
 	}
 
-	return ucma_complete(id_priv);
+	return ucma_complete(id);
 }
 
 int rdma_listen(struct rdma_cm_id *id, int backlog)
@@ -1450,7 +1465,7 @@ int rdma_accept(struct rdma_cm_id *id, struct rdma_conn_param *conn_param)
 	if (ucma_is_ud_qp(id->qp_type))
 		return 0;
 
-	return ucma_complete(id_priv);
+	return ucma_complete(id);
 }
 
 int rdma_reject(struct rdma_cm_id *id, const void *private_data,
@@ -1521,7 +1536,7 @@ int rdma_disconnect(struct rdma_cm_id *id)
 	if (ret != sizeof cmd)
 		return (ret >= 0) ? ERR(ENODATA) : -1;
 
-	return ucma_complete(id_priv);
+	return ucma_complete(id);
 }
 
 static int rdma_join_multicast2(struct rdma_cm_id *id, struct sockaddr *addr,
@@ -1583,7 +1598,7 @@ static int rdma_join_multicast2(struct rdma_cm_id *id, struct sockaddr *addr,
 	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
 
 	mc->handle = resp.id;
-	return ucma_complete(id_priv);
+	return ucma_complete(id);
 
 err2:
 	pthread_mutex_lock(&id_priv->mut);
@@ -2155,7 +2170,7 @@ int rdma_create_ep(struct rdma_cm_id **id, struct rdma_addrinfo *res,
 		ret = rdma_set_option(cm_id, RDMA_OPTION_IB, RDMA_OPTION_IB_PATH,
 				      res->ai_route, res->ai_route_len);
 		if (!ret)
-			ret = ucma_complete(container_of(cm_id, struct cma_id_private, id));
+			ret = ucma_complete(cm_id);
 	} else {
 		ret = rdma_resolve_route(cm_id, 2000);
 	}
@@ -2205,3 +2220,11 @@ void rdma_destroy_ep(struct rdma_cm_id *id)
 
 	rdma_destroy_id(id);
 }
+
+int ucma_max_qpsize(struct rdma_cm_id *id)
+{
+	struct cma_id_private *id_priv;
+
+	id_priv = container_of(id, struct cma_id_private, id);
+	return id_priv->cma_dev->max_qpsize;
+}
